# legacy.
#!/bin/sh

curl http://download.tensorflow.org/models/resnet_v1_50_2016_08_28.tar.gz | tar xvz
curl http://download.tensorflow.org/models/resnet_v1_101_2016_08_28.tar.gz | tar xvz
curl http://download.tensorflow.org/models/resnet_v1_152_2016_08_28.tar.gz | tar xvz